# copyright 2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-seda ITreeBase adapters"""

from cubicweb.predicates import is_instance
from cubicweb.view import EntityAdapter

from ..xsd2yams import MULTIPLE_CHILDREN
from . import simplified_profile


def parent_archive_unit(entity):
    """Return the first encountered parent which is an ArchiveUnit"""
    while entity.cw_etype != 'SEDAArchiveUnit':
        entity = entity.cw_adapt_to('IContained').parent
    return entity


class IContainedToITreeBase(EntityAdapter):
    """Map IContained adapters to ITreeBase, additionaly configured with a list of relations leading to
    contained's children.

    ITreeBase is a simplified version of cubicweb's ITree.
    """

    __regid__ = 'ITreeBase'
    __abstract__ = True

    _children_relations = []  # list of (relation type, role) to get entity's children

    def iterchildren(self):
        """Return an iterator over the item's children."""
        for rel, role in self._children_relations:
            for child in self.entity.related(rel, role, entities=True):
                yield child

    def is_leaf(self):
        """Returns True if the entity does not have any children."""
        try:
            next(iter(self.iterchildren()))
            return False
        except StopIteration:
            return True

    def parent(self):
        return self.entity.cw_adapt_to('IContained').parent

    def iterancestors(self):
        """Return an iterator on the ancestors of the entity."""
        def _uptoroot(self):
            curr = self
            while True:
                curr = curr.parent()
                if curr is None:
                    break
                yield curr
                curr = curr.cw_adapt_to('ITreeBase')
        return _uptoroot(self)

    iterparents = iterancestors  # Compat. with CubicWeb's ITree.


class ITreeBaseArchiveUnitAdapter(IContainedToITreeBase):
    """Adapt ArchiveUnit entities to ITreeBase."""

    __select__ = is_instance('SEDAArchiveUnit')

    _children_relations = [('seda_archive_unit', 'object')]

    def parent(self):
        parent = self.entity.cw_adapt_to('IContained').parent
        if parent is None:
            return None
        if parent.cw_etype == 'SEDAArchiveTransfer':
            return parent
        return parent_archive_unit(parent)

    def iterchildren(self):
        seq = self.entity.first_level_choice.content_sequence
        if seq is None:
            # 'reference' archive unit
            return
        for rel, role in self._children_relations:
            for child in seq.related(rel, role, entities=True):
                yield child


class ITreeBaseSimplifiedArchiveUnitAdapter(ITreeBaseArchiveUnitAdapter):

    __select__ = ITreeBaseArchiveUnitAdapter.__select__ & simplified_profile()

    _children_relations = [('seda_archive_unit', 'object')]

    def iterchildren(self):
        for child in super(ITreeBaseSimplifiedArchiveUnitAdapter, self).iterchildren():
            yield child
        seq = self.entity.first_level_choice.content_sequence
        assert seq is not None, self.entity  # can't be None in simplified profile
        for do in self._cw.execute(
                'Any DO, DOUA ORDERBY REFO WHERE DO user_annotation DOUA, REF ordering REFO, '
                'REF seda_data_object_reference_id DO, '
                'REF seda_data_object_reference SEQ, SEQ eid %(x)s',
                {'x': seq.eid}).entities():
            yield do


class ITreeBaseDataObjectAdapter(IContainedToITreeBase):
    """Adapt BinaryDataObject and PhysicalDataObject entities to ITreeBase."""

    __select__ = is_instance('SEDABinaryDataObject', 'SEDAPhysicalDataObject')


class ITreeBaseSimplifiedDataObjectAdapter(ITreeBaseDataObjectAdapter):

    __select__ = ITreeBaseDataObjectAdapter.__select__ & simplified_profile()

    def parent(self):
        return parent_archive_unit(self.entity.reverse_seda_data_object_reference_id[0])


class ITreeBaseArchiveTransferAdapter(IContainedToITreeBase):
    """Adapt ArchiveTransfer entities to ITreeBase."""

    __select__ = is_instance('SEDAArchiveTransfer')

    @property
    def _children_relations(self):
        if self.entity.simplified_profile:
            return [('seda_archive_unit', 'object')]
        else:
            return [('seda_binary_data_object', 'object'),
                    ('seda_physical_data_object', 'object'),
                    ('seda_archive_unit', 'object')]

    def parent(self):
        return None


# Tree ordering management
# ------------------------
#
# when several child entities of the same types are allowed, we want to control
# their relative order. This is done using the `ordering` attribute of the child
# entities.
#
# Code below handle moving nodes among the tree. As such it has to handle this
# attribute so that:
#
# * move may control order
#
# * reparenting will keep consistent order
#
# In order to minimize the number of queries and to keep code clearer (avoid
# transmitting ordering number through json among other), I've (syt) choosen
# that `ordering` attribute's value:
#
# * starts at `1` for the first element,
#
# * is a sequential list, i.e. `2` for the 2nd child, `3` for the 3rd child, and
#   up to `len(children)` for the last one.

ETYPE_PARENT_RTYPE = dict(MULTIPLE_CHILDREN)


def next_child_ordering(cnx, parent_eid, rtype):
    """Return value for the `ordering` attribute of a child freshly appended through
    `rtype` to parent entity with the given eid.
    """
    rql = 'Any MAX(O) WHERE X {rtype} P, P eid %(p)s, X ordering O'
    ordering = cnx.execute(rql.format(rtype=rtype), {'p': parent_eid})[0][0]
    return 1 if ordering is None else ordering + 1


def move_child_at_index(cnx, parent_eid, child_eid, index, reparenting=False):
    """Given a parent node and one of its child, move it at `index` position (0 for
    the first position, and `len(children)` for the last).

    `reparenting` should be true when the child has just been appended, hence
    it's previous ordering should not be considered.
    """
    child = cnx.entity_from_eid(child_eid)
    assert reparenting or child.ordering not in (index, index + 1), \
        "moving a child before or after itself doesn't make sense"
    rtype = ETYPE_PARENT_RTYPE[child.cw_etype]
    if reparenting:
        rql = ('SET X ordering XO + 1 WHERE X ordering XO, '
               'X ordering > %(index)s, '
               'X {rtype} P, P eid %(p)s'.format(rtype=rtype))
        order = index + 1
    elif child.ordering > index:
        rql = ('SET X ordering XO + 1 WHERE X ordering XO, '
               'X ordering > %(index)s, X ordering < %(porder)s, '
               'X {rtype} P, P eid %(p)s'.format(rtype=rtype))
        order = index + 1
    else:
        rql = ('SET X ordering XO - 1 WHERE X ordering XO, '
               'X ordering <= %(index)s, X ordering > %(porder)s, '
               'X {rtype} P, P eid %(p)s'.format(rtype=rtype))
        order = index
    cnx.execute(rql, {'index': index,
                      'porder': None if reparenting else child.ordering,
                      'p': parent_eid}).rows
    child.cw_set(ordering=order)


def prepare_child_removal(child):
    """Before `child` will be removed or reparented, update its former parent's
    child to keep their `ordering` attribute sequential.

    This is expected to be called by a hook.
    """
    rtype = ETYPE_PARENT_RTYPE[child.cw_etype]
    child._cw.execute(
        'SET C ordering CO - 1 WHERE C ordering CO, C ordering > XO, '
        'X ordering XO, X {rtype} P, C {rtype} P, X eid %(x)s'.format(rtype=rtype),
        {'x': child.eid}).rows


def reparent(entity, new_parent_eid, index):
    """Move `entity` as a children of `new_parent_eid` at `index` position.
    """
    rtype = ETYPE_PARENT_RTYPE[entity.cw_etype]
    entity.cw_set(**{rtype: new_parent_eid})
    move_child_at_index(entity._cw, new_parent_eid, entity.eid, index,
                        reparenting=True)



from cubicweb_seda.dataimport import import_seda_schemes

rql('SET X compat_list CL + ", RNG" WHERE X compat_list CL')
commit(ask_confirm=False)

rql('INSERT SEDAMimeType X: X user_cardinality "0..1", X seda_mime_type_from BDO '
    'WHERE NOT Y seda_mime_type_from BDO')

rql('INSERT SEDAFormatId X: X user_cardinality "0..1", X seda_format_id_from BDO '
    'WHERE NOT Y seda_format_id_from BDO')
commit(ask_confirm=False)

sync_schema_props_perms('seda_mime_type_to')
sync_schema_props_perms('seda_format_id_to')
sync_schema_props_perms('seda_data_object_reference_id')

rql('DELETE CS scheme_relation_type RT WHERE '
    'RT name IN ("seda_language_to", "seda_description_language_to")')
commit(ask_confirm=False)
import_seda_schemes(cnx)



from cubicweb_seda.dataimport import import_seda_schemes, LCSV_FILES

add_entity_type('SEDAMessageDigestAlgorithmCodeListVersion')
drop_relation_type('seda_message_digest_algorithm_code_list_version')
add_entity_type('SEDAFileFormatCodeListVersion')
drop_relation_type('seda_file_format_code_list_version')

add_entity_type('SEDAAcquisitionInformationCodeListVersion')
add_entity_type('SEDAAcquisitionInformation')

add_entity_type('SEDAOriginatingAgencyIdentifier')
add_entity_type('SEDASubmissionAgencyIdentifier')

add_entity_type('SEDALegalStatus')

sync_schema_props_perms('seda_algorithm')
sync_schema_props_perms('seda_format_id_to')

for etype in ('SEDArestrictionRuleIdRef',
              'SEDArestrictionValue',
              'SEDArestrictionEndDate'):
    drop_entity_type(etype)

lcsv_files = [file_def for file_def in LCSV_FILES
              if file_def[-1] in ('legal_status.csv', 'digest_algorithms.csv', 'dissemination.csv')]
import_seda_schemes(cnx, lcsv_files=lcsv_files)

drop_relation_type('seda_reply_code_list_version')
add_relation_definition('SEDAArchiveTransfer', 'clone_of', 'SEDAArchiveTransfer')



# copyright 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This file is part of CubicWeb.
#
# CubicWeb is free software: you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# CubicWeb is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with CubicWeb.  If not, see <http://www.gnu.org/licenses/>.
"""

"""
import os
import sys

from logilab.common.pytest import PyTester


def getlogin():
    """avoid usinng os.getlogin() because of strange tty / stdin problems
    (man 3 getlogin)
    Another solution would be to use $LOGNAME, $USER or $USERNAME
    """
    if sys.platform == 'win32':
        return os.environ.get('USERNAME') or 'cubicweb'
    import pwd
    return pwd.getpwuid(os.getuid())[0]


def update_parser(parser):
    login = getlogin()
    parser.add_option('-r', '--rebuild-database', dest='rebuild_db',
                      default=False, action="store_true",
                      help="remove tmpdb and rebuilds the test database")
    parser.add_option('-u', '--dbuser', dest='dbuser', action='store',
                      default=login, help="database user")
    parser.add_option('-w', '--dbpassword', dest='dbpassword', action='store',
                      default=login, help="database user's password")
    parser.add_option('-n', '--dbname', dest='dbname', action='store',
                      default=None, help="database name")
    parser.add_option('--euser', dest='euser', action='store',
                      default=login, help="euser name")
    parser.add_option('--epassword', dest='epassword', action='store',
                      default=login, help="euser's password' name")
    return parser


class CustomPyTester(PyTester):
    def __init__(self, cvg, options):
        super(CustomPyTester, self).__init__(cvg, options)
        if options.rebuild_db:
            os.unlink('tmpdb')
            os.unlink('tmpdb-template')



# copyright 2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-seda adapter classes for generation of a profile generation as HTML"""

from six import string_types

from cubicweb import _

from .profile_generation import SEDA2ExportAdapter, content_types
from .profile_generation import xselement_scheme_attribute, _concept_value


def element_uml_cardinality(occ, card_entity):
    """Return UML like cardinality for the given pyxst Occurence. Cardinality may be
    overriden by the data model's user_cardinality value.
    """
    cardinality = getattr(card_entity, 'user_cardinality', None)
    if cardinality is None:
        minimum = occ.minimum
        maximum = occ.maximum
        if minimum == maximum == 1:
            return '1'
        elif maximum != 1:
            maximum = 'n'
        return '%s..%s' % (minimum, maximum)
    else:
        return cardinality


def attribute_cardinality_as_string(occ, card_entity):
    """Return 'optional' or 'mandatory' for the given pyxst attribute's Occurence. Cardinality may be
    overriden by the data model's user_cardinality value.
    """
    cardinality = getattr(card_entity, 'user_cardinality', None)
    if cardinality is None:
        minimum = occ.minimum
    else:
        # XXX assert cardinality in ('0..1', '1'), cardinality
        if cardinality[0] == '0':
            minimum = 0
        else:
            minimum = 1
    return _('mandatory') if minimum else _('optional')


class SEDA2HTMLExport(SEDA2ExportAdapter):
    """Adapter to build a HTML representation of a SEDA profile"""
    __regid__ = 'SEDA-2.0.html'
    namespaces = {}
    css = '''
    h1{
      font-variant: small-caps;
      color : #FF9800;
      text-align: center;
      padding : 10px;
      font-weight: lighter;
      border-bottom : 2px solid  #E0E0E0;
    }

    a{
      color : #EF6C00;
    }
    body {
      font-family: helvetica, Arial;
      background-color: #FAFAFA;
      color : #666;
    }
    body >div{
      background-color: #FFF;
      padding-bottom : 20px;
      margin-top : 20px;
      box-shadow: 2px 2px 2px 0px #E0E0E0;
    }

    body >div >h3{
      font-variant: normal;
      font-weight: lighter;
      font-size:15px;
      padding : 10px;
      background-color: #EF6C00;
      color : white;
      font-variant: small-caps;
      cursor : pointer;
    }

    body >div >h3 span.card{
      font-size: 70%;
      color : #E0E0E0;
      margin-left : 5px;
    }

    body>div>div, .sequence >div{
      margin-top : 20px
    }

    body>div>div div{
      padding-left : 20px;
      margin-left: 15px;
      border-left : 1px dashed #EF6C00;
    }

    body>div>div h3{
      margin-bottom : 10px;
    }

    h3 span.card {
      font-size: 70%;
      color : #EF6C00;
      margin-left : 5px;
    }

    span {
      font-size: 90%;
      color: #555;
    }
    span.label {
      font-weight: bold;
      padding-left: 1px;
    }

    div.attribute {
      color: #888;
      width: 40em;
      font-size: small;
      padding : 10px;
    }

    div.attribute span.label {
      float: left;
      color : #EF6C00;
      font-weight: lighter;
      margin-right : 20px;
    }

    .sequence >h3:first-child{
      color :  #EF6C00;
    }
    '''

    def dump_etree(self):
        """Return an XSD etree for the adapted SEDA profile."""
        root = self.element('html')
        head = self.element('head', root)
        self.element('title', head, text=self.entity.dc_title())
        self.element('meta', head, {'http-equiv': 'content-type',
                                    'content': 'text/html; charset=UTF-8'})
        self.element('style', head, text=self.css)
        body = self.element('body', root)
        self._dump(body)
        return root

    def init_transfer_element(self, xselement, root, entity):
        self.element('h1', root, text=entity.title)
        if entity.user_annotation:
            self.element('div', root, text=entity.user_annotation,
                         attributes={'class': 'description'})
        return root

    def jumped_element(self, profile_element):
        return profile_element[-1]

    def element_alternative(self, occ, profile_element, target_value, to_process, card_entity):
        div = self.element('div', profile_element,
                           attributes={'class': 'choice'})
        self.title(div, self._cw._('Alternative'), occ, card_entity)
        to_process[occ.target].append((target_value, div))

    def element_sequence(self, occ, profile_element, target_value, to_process, card_entity):
        div = self.element('div', profile_element,
                           attributes={'class': 'sequence'})
        self.title(div, self._cw._('Sequence'), occ, card_entity)
        to_process[occ.target].append((target_value, div))

    def element_xmlattribute(self, occ, profile_element, target_value, to_process, card_entity):
        div = self.element('div', profile_element,
                           attributes={'class': 'attribute'})
        card = self._cw._(attribute_cardinality_as_string(occ, card_entity))
        self.element('span', div, text=occ.target.local_name, attributes={'class': 'label'})
        self.element('span', div, text=card, attributes={'class': 'card'})
        fixed_value = self.serialize(target_value)
        if isinstance(fixed_value, string_types):
            self.element('span', div, text=fixed_value, attributes={'class': 'value'})
        else:
            span = self.element('span', div, attributes={'class': 'value'})
            if fixed_value is not None:
                span.append(fixed_value)

    def element_xmlelement(self, occ, profile_element, target_value, to_process, card_entity):
        xselement = occ.target
        if isinstance(occ, dict):  # fake occurence introduced for some elements'content
            return
        attrs = {}
        if hasattr(target_value, 'id'):
            attrs['id'] = target_value.id
        div = self.element('div', profile_element, attrs)
        self.title(div, xselement.local_name, occ, card_entity)
        annotation = getattr(card_entity, 'user_annotation', None)
        if annotation:
            self.element('div', div, text=annotation,
                         attributes={'class': 'description'})
        xstypes = content_types(xselement.textual_content_type)
        self.fill_element(xselement, div, target_value, card_entity, xstypes)
        if getattr(target_value, 'eid', None):  # value is an entity
            to_process[xselement].append((target_value, div))

    def fill_element(self, xselement, div, target_value, card_entity, xstypes=None):
        if xselement.local_name == 'KeywordType':
            list_div = self.element('div', div)
            self.element('span', list_div, text=u'listVersionID',
                         attributes={'class': 'label'})
            if target_value:
                list_value = target_value.scheme.description or target_value.scheme.dc_title()
            else:
                list_value = 'edition 2009'
            self.element('span', list_div, text=list_value, attributes={'class': 'value'})

        elif (xselement.local_name == 'KeywordReference' and card_entity.scheme):
            self.concept_scheme_attribute(xselement, div, card_entity.scheme)

        elif getattr(target_value, 'cw_etype', None) == 'Concept':
            self.concept_scheme_attribute(xselement, div, target_value.scheme)
        if xstypes:
            ct_div = self.element('div', div)
            self.element('span', ct_div, text=self._cw._('XSD content type'),
                         attributes={'class': 'label'})
            xstypes = self._cw._(' ALT_I18N ').join(u'xsd:' + xstype for xstype in xstypes)
            self.element('span', ct_div, text=xstypes, attributes={'class': 'value'})
        if target_value is None:
            values = ()
        elif not isinstance(target_value, (tuple, list)):
            values = [target_value]
        else:
            values = target_value
        value_div = None
        for value in values:
            fixed_value = self.serialize(value)
            if fixed_value is None:
                continue
            if value_div is None:
                value_div = self.element('div', div)
                self.element('span', value_div, text=self._cw._('fixed value'),
                             attributes={'class': 'label'})
            if isinstance(fixed_value, string_types):
                self.element('span', value_div, text=fixed_value, attributes={'class': 'value'})
            else:
                span = self.element('span', value_div, attributes={'class': 'value'})
                span.append(fixed_value)

    def concept_scheme_attribute(self, xselement, div, scheme):
        try:
            xsattr = xselement_scheme_attribute(xselement)
        except KeyError:
            return  # no attribute to define scheme (e.g. for language specification)
        div = self.element('div', div)
        self.element('span', div, text=xsattr, attributes={'class': 'label'})
        self.element('a', self.element('span', div, attributes={'class': 'value'}),
                     {'href': scheme.absolute_url()}, text=scheme.dc_title())

    def title(self, div, label, occ, card_entity):
        h3 = self.element('h3', div, text=label)
        self.element('span', h3, attributes={'class': 'card'},
                     text=u'[{0}]'.format(self._cw._(element_uml_cardinality(occ, card_entity))))

    def serialize(self, value):
        """Return value as None or some text or etree node to be inserted in the HTML DOM."""
        if value is None:
            return None
        if hasattr(value, 'eid'):
            if value.cw_etype in ('AuthorityRecord', 'ConceptScheme'):
                href, label = value.absolute_url(), value.dc_title()
            elif value.cw_etype == 'Concept':
                href = value.absolute_url()
                label = _concept_value(value, 'seda-2')
                if value.label() != label:
                    label = u'{0} ({1})'.format(label, value.label())
            elif hasattr(value, 'id'):
                # value is something in the profile which has a id (e.g. archive unit, data object)
                href, label = u'#{0}'.format(value.id), value.id
            else:
                return None  # intermediary entity
            return self.element('a', self.element('span', attributes={'class': 'value'}),
                                attributes={'href': href}, text=label)
        else:
            if isinstance(value, bool):
                value = 'true' if value else 'false'
            assert isinstance(value, string_types), repr(value)
            return value



sync_schema_props_perms('language_code')

scheme = cnx.find('ConceptScheme', title=u'Langues (ISO-639-3)').one()
with cnx.deny_all_hooks_but('metadata'):
    rset = cnx.execute(
        'Any C, L, LL WHERE L label_of C, L language_code "seda-2", L label LL,'
        'C in_scheme CS, CS eid %(cs)s', {'cs': scheme.eid})
    for i, concept in enumerate(rset.entities()):
        label = rset.get_entity(i, 1)
        label.cw_set(language_code=u'seda')
        cnx.create_entity('Label', label_of=concept, language_code='seda-02',
                          label=label.label[:2])
    commit()




from __future__ import print_function
from cubicweb_seda.dataimport import LCSV_FILES, import_seda_schemes

# please close your eyes - this is necessary if seda is used by saem else call
# to import_seda_schemes will fail because it will attempt to reuse and existing
# uri. Should be done there but still easier here.
try:
    from cubicweb_saem_ref.site_cubicweb import init_seda_scheme
    init_seda_scheme.func_defaults[0][0] = len(LCSV_FILES) - 1
except ImportError:
    pass

print('filtering language concepts, this will take a while')

seda_02_langs = set((
    'aa', 'ab', 'af', 'ak', 'am', 'ar', 'an', 'as', 'av', 'ae', 'ay', 'az',
    'ba', 'bm', 'be', 'bn', 'bh', 'bi', 'bo', 'bs', 'br', 'bg',
    'ca', 'cs', 'ch', 'ce', 'cu', 'cv', 'co', 'cr', 'cy',
    'da', 'de', 'dv', 'dz',
    'el', 'en', 'eo', 'et', 'ee', 'es', 'eu',
    'fo', 'fa', 'fj', 'fi', 'fr', 'fy', 'ff',
    'ga', 'gl', 'gv', 'gn', 'gu',
    'ht', 'ha', 'he', 'hz', 'hi', 'ho', 'hr', 'hu', 'hy',
    'ig', 'is', 'io', 'ii', 'iu', 'ie', 'ia', 'id', 'ik', 'it',
    'jv', 'ja',
    'ka', 'kl', 'kn', 'ks', 'kr', 'kk', 'km', 'ki', 'ky', 'kv', 'kg', 'ko', 'kj', 'ku', 'kw',
    'lo', 'lv', 'li', 'ln', 'lt', 'lb', 'lu', 'lg',
    'mk', 'mh', 'ml', 'mi', 'mr', 'ms', 'mg', 'mt', 'mn', 'my',
    'na', 'nv', 'nr', 'nd', 'ng', 'ne', 'nn', 'nb', 'no', 'ny', 'nl',
    'oc', 'oj', 'or', 'om', 'os',
    'pa', 'pi', 'pl', 'pt', 'ps'
    'qu',
    'rm', 'ro', 'rn', 'ru', 'rw',
    'sg', 'sa', 'si', 'sk', 'sl', 'se', 'sm', 'sn', 'sd', 'so', 'st', 'sc', 'sr', 'ss', 'su', 'sw', 'sv', 'sq',
    'ty', 'ta', 'tt', 'te', 'tg', 'tl', 'th', 'ti', 'to', 'tn', 'ts', 'tk', 'tr', 'tw',
    'ug', 'uk', 'ur', 'uz',
    've', 'vi', 'vo',
    'wa', 'wo',
    'xh',
    'yi', 'yo',
    'za', 'zu', 'zh',
))

rset = rql(
    'Any X WHERE X in_scheme S, S title "Langues (ISO-639-3)", '
    'L label_of X, L language_code "seda-02", NOT L label IN ({})'
    .format(','.join(repr(code) for code in seda_02_langs)))
print(len(rset), 'concepts to delete')
for i, concept in enumerate(rset.entities()):
    concept.cw_delete()
    if i % 10:
        cnx.commit()


sync_schema_props_perms('user_cardinality')

add_relation_type('file_category')

import_seda_schemes(cnx)

sync_schema_props_perms('seda_mime_type_to')
sync_schema_props_perms('seda_format_id_to')



# copyright 2015-2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-seda custom fields/widgets"""

from six import text_type

from logilab.common.decorators import monkeypatch

from cubicweb import tags, utils
from cubicweb.uilib import js
from cubicweb.web import formfields as ff, formwidgets as fw
from cubicweb.web.views import ajaxcontroller, autoform, formrenderers

from cubes.relationwidget import views as rwdg


# deactivate relation widget's creation form by default, it causes some js error if e.g. there are
# some calendar widgets. Also, we don't really want that from an UX POV, IMO.
rwdg.SearchForRelatedEntitiesView.has_creation_form = False


def configure_relation_widget(req, div, search_url, title, multiple, validate):
    """Build a javascript link to invoke a relation widget

    Widget will be linked to div `div`, with a title `title`. It will display selectable entities
    matching url `search_url`. bool `multiple` indicates whether several entities can be selected or
    just one, `validate` identifies the javascript callback that must be used to validate the
    selection.
    """
    req.add_js(('jquery.ui.js',
                'cubicweb.ajax.js',
                'cubicweb.widgets.js',
                'cubicweb.facets.js',
                'cubes.relationwidget.js',
                'cubes.editionext.js'))
    return 'javascript: %s' % js.editext.relateWidget(div, search_url, title, multiple,
                                                      utils.JSString(validate))


class ConceptOrTextField(ff.Field):
    """Compound field, using the :class:`ConceptAutoCompleteWidget` to handle values which may be
    either a relation to a `Concept` or a string

    It's expected to be set on the text attribute. Also, as you should prefer specifying a field's
    class, you will usually sublcass this field to set the following attributes:

    * 'scheme_relation', relation whose object will be the selected scheme
    * 'concept_relation', relation whose object will be the linked concept

    Example::

        class KeywordConceptOrTextField(ConceptOrTextField):
            scheme_relation = 'keyword_scheme'
            concept_relation = 'keyword_concept'

            def __init__(self, **kwargs):
                super(EquivalentConceptOrTextField, self).__init__(required=True, **kwargs)
                self.help = _('when linked to a vocabulary, value is enforced to the label of a '
                              'concept in this vocabulary. Remove it if you want to type text '
                              'freely.')

        aff = uicfg.autoform_field
        aff.tag_attribute(('Keyword', 'keyword_value'), KeywordConceptOrTextField)

    You may also configure the ajax function that will be used for the completion values, by
    specifying it using the 'ajax_autocomplete_func' attribute (or by overriding this ajax
    func). Default is :func:`scheme_concepts_autocomplete`.
    """

    scheme_relation = concept_relation = None
    ajax_autocomplete_func = 'scheme_concepts_autocomplete'

    def get_widget(self, form):
        return ConceptAutoCompleteWidget(self.name, self.scheme_relation,
                                         self.ajax_autocomplete_func, optional=True)

    def has_been_modified(self, form):
        return True  # handled in process_posted below

    def process_posted(self, form):
        posted = form._cw.form
        text_val = posted.get(self.input_name(form, 'Label'), '').strip()
        equivalent_eid = posted.get(self.input_name(form), '').strip()
        equivalent_eids = set()
        if equivalent_eid:
            equivalent_eids.add(int(equivalent_eid))
        if self.required and not (text_val or equivalent_eid):
            raise ff.ProcessFormError(form._cw.__("required field"))
        entity = form.edited_entity
        if not entity.has_eid() or getattr(entity, self.name) != text_val:
            yield (ff.Field(name=self.name, role='subject', eidparam=True), text_val)
        if (not entity.has_eid() and equivalent_eids) \
           or (entity.has_eid()
               and set(x.eid for x in entity.equivalent_concept) != equivalent_eids):
            subfield = ff.Field(name=self.concept_relation, role='subject', eidparam=True)
            # register the association between the value and the field, because on entity creation,
            # process_posted will be recalled on the newly created field, and if we don't do that it
            # won't find the proper value (this is very nasty)
            form.formvalues[(subfield, form)] = equivalent_eids
            yield (subfield, equivalent_eids)


class KeywordTypeMasterWidget(fw.Select):
    """
    Usage::

      affk = uicfg.autoform_field_kwargs
      affk.set_field_kwargs('KeywordType', 'keyword_type_to',
                            widget=widgets.KeywordTypeMasterWidget(
                            slave_base_name='seda_keyword_reference_to_scheme'))
    """

    def __init__(self, slave_base_name, **kwargs):
        super(KeywordTypeMasterWidget, self).__init__(**kwargs)
        self.slave_base_name = slave_base_name

    def _render(self, form, field, render):
        req = form._cw

        vocabularies_data = []
        for eid, title, uri, keyword_type in req.execute(
                'Any CS,CST,CSU,CSKT WHERE CS title CST, CS cwuri CSU, CS code_keyword_type CSKT?'):
            vocabularies_data.append({'eid': eid, 'title': title or uri,
                                      'keyword_type': keyword_type})
        vocabularies_data.sort(key=lambda x: x['title'])

        req.add_js(('cubicweb.js', 'cubicweb.ajax.js', 'cubes.skoscomplete.js'))
        req.add_onload(js.typed_vocabularies.initKeywordTypeMasterWidget(
            field.dom_id(form, self.suffix), self.slave_base_name, vocabularies_data))

        return super(KeywordTypeMasterWidget, self)._render(form, field, render)


class ConceptAutoCompleteWidget(fw.TextInput):
    """Derive from simple text input to create an autocompletion widget if a scheme is specified,
    otherwise free text is fine if `optional` argument is true. In such case:

    * `slave_name` is expected to be the name of the concept attribute, or the text attribute if
      optional is true),

    * you'll have to use this widget from a custom field that will handle the relation to the
      concept (e.g. :class:`ConceptOrTextField`).

    When optional is false, a regular :class:`RelationField` is fine.

    You may configure the ajax function that will be used for the completion values by specifying it
    using the 'ajax_autocomplete_func' argument (or by overriding this ajax func). Default is
    :func:`scheme_concepts_autocomplete`.

    Usage::

      affk = uicfg.autoform_field_kwargs
      affk.set_field_kwargs('Keyword', 'keyword_value',
                            widget=ConceptAutoCompleteWidget(slave_name='keyword_value',
                                                             master_name='keyword_scheme',
                                                             optional=True))
    """
    needs_css = ('jquery.ui.css',)
    needs_js = ('jquery.ui.js',
                'cubicweb.js', 'cubicweb.ajax.js',
                'cubes.skoscomplete.js')

    def __init__(self, slave_name, master_name,
                 ajax_autocomplete_func='scheme_concepts_autocomplete',
                 optional=False,
                 **kwargs):
        super(ConceptAutoCompleteWidget, self).__init__(**kwargs)
        self.slave_name = slave_name
        self.master_name = master_name
        self.ajax_autocomplete_func = ajax_autocomplete_func
        self.optional = optional

    def _render(self, form, field, render):
        entity = form.edited_entity
        slave_id = field.dom_id(form, self.suffix)
        master_id = slave_id.replace(self.slave_name, self.master_name)
        if entity.has_eid():
            concept = entity.concept
        else:
            concept = None
        req = form._cw
        req.add_onload(js.concept_autocomplete.initConceptAutoCompleteWidget(
            master_id, slave_id, self.ajax_autocomplete_func))
        if concept is None:
            value = getattr(entity, self.slave_name) if self.optional else None
            eid = u''
        else:
            value = concept.label()
            eid = text_type(concept.eid)
        # we need an hidden input to handle the value while the text input display the label
        inputs = [
            tags.input(name=field.input_name(form, 'Label'), id=slave_id + 'Label',
                       klass='form-control', type='text',
                       value=value),
            tags.input(name=field.input_name(form), id=slave_id, type='hidden',
                       value=eid)
        ]
        return u'\n'.join(inputs)


@ajaxcontroller.ajaxfunc(output_type='json')
def scheme_concepts_autocomplete(self):
    assert self._cw.form['scheme']
    scheme = int(self._cw.form['scheme'])
    term = self._cw.form['q']
    limit = self._cw.form.get('limit', 50)
    return [{'value': eid, 'label': label}
            for eid, label in self._cw.execute(
                'DISTINCT Any C,N ORDERBY N LIMIT %s WHERE C in_scheme S, S eid %%(s)s, '
                'C preferred_label L, L label N, L label ILIKE %%(term)s' % limit,
                {'s': scheme, 'term': u'%%%s%%' % term})]


@monkeypatch(formrenderers.EntityInlinedFormRenderer)
def render_title(self, w, form, values):
    """Monkey-patched to remove counter"""
    w(u'<div class="iformTitle">')
    w(u'<span>%(title)s</span>' % values)
    if values['removejs']:
        values['removemsg'] = self._cw.__('remove-inlined-entity-form')
        w(u' [<a href="javascript: %(removejs)s;$.noop();">%(removemsg)s</a>]'
          % values)
    w(u'</div>')


class SEDAMetaField(ff.StringField):
    """Extends user_cardinality field to allow edition of a `user_annotation` all together.
    """

    @staticmethod
    def annotation_field(form):
        """Return the field for `user_annotation` entity. """
        eschema = form.edited_entity.e_schema
        rschema = eschema.subjrels['user_annotation']
        return ff.guess_field(eschema, rschema, 'subject', form._cw)

    def actual_fields(self, form):
        """Overriden from :class:`Field`"""
        yield self
        yield self.annotation_field(form)

    def render(self, form, renderer):
        """Overriden from :class:`Field` to render fields in a div which is hidden by default"""
        form._cw.add_js('cubes.seda.js')
        wdgs = [self.get_widget(form).render(form, self, renderer)]
        self._render_hidden_section(wdgs, form, renderer)
        return u'\n'.join(wdgs)

    def _render_hidden_section(self, wdgs, form, renderer):
        divid = '%s-advanced' % self.input_name(form)
        if form.edited_entity.has_eid() and form.edited_entity.user_annotation:
            hidden = ''
            icon = 'icon-up-open'
        else:
            hidden = ' hidden'
            icon = 'icon-list-add'
        wdgs.append(tags.a(u'', onclick=text_type(js.seda.toggleFormMetaVisibility(divid)),
                           href='javascript:$.noop()', title=form._cw._('show/hide meta fields'),
                           # take care, related js relies on the icon class position
                           klass=icon + ' metaFieldSwitch'))
        wdgs.append(u'<div id="{0}" class="metaField{1}">'.format(divid, hidden))
        wdgs.append(self._render_subfield(form, self.annotation_field(form), renderer))
        wdgs.append(u'</div>')

    @staticmethod
    def _render_subfield(form, field, renderer):
        """Render a sub-field: label + widget + help + EOL"""
        data = utils.UStringIO()
        w = data.write
        w(u'<div class="row">')
        w(renderer.render_label(form, field))
        w(u'<div class="col-md-9">')
        w(field.render(form, renderer))
        w(u'</div>')
        w(u'</div>')
        w(u'<div class="row">')
        w(u'<div class="col-md-offset-3 col-md-9">')
        w(renderer.render_help(form, field))
        w(u'</div>')
        w(u'</div>')
        return data.getvalue()


class SimplifiedAutomaticEntityForm(autoform.AutomaticEntityForm):
    """Custom autoform, forcing display of creation form instead of add new link.
    """

    __abstract__ = True

    def should_display_inline_creation_form(self, rschema, existing, card):
        return not existing

    def should_display_add_new_relation_link(self, rschema, existing, card):
        return False


class NoTitleEntityInlinedFormRenderer(formrenderers.EntityInlinedFormRenderer):
    """Custom inlined form renderer that doesn't display any title nor remove link.

    This is intended to be subclassed with a custom selector.
    """

    __abstract__ = True

    def render_title(self, w, form, values):
        pass


class ConcretNoTitleEntityInlinedFormRenderer(NoTitleEntityInlinedFormRenderer):
    """Concret implementation of `NoTitleEntityInlinedFormRenderer` with a custom regid.

    Use this one by specifying the renderer id explicitly, for case where you can't easily specify a
    selector.
    """
    __regid__ = 'notitle'



scheme = cnx.find('ConceptScheme', title=u'Types MIME').one()
sql("""
UPDATE cw_label SET cw_language_code = 'seda'
FROM cw_concept, in_scheme_relation
WHERE in_scheme_relation.eid_to = {}
AND cw_concept.cw_eid = in_scheme_relation.eid_from
AND cw_label_of = cw_concept.cw_eid
AND cw_language_code = 'seda-2'
""".format(scheme.eid))




# copyright 2015-2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-seda views related to XSD download"""

from logilab.common.registry import objectify_predicate

from cubicweb import view, _
from cubicweb.predicates import is_instance, one_line_rset
from cubicweb.web import httpcache, action
from cubicweb.web.views import idownloadable


@objectify_predicate
def format_supported(cls, req, rset=None, entity=None, **kwargs):
    """Predicate matching cases where an expected format (specified as `version` form param or as a
    `seda_version` class attribute on the appobject) is supported by the context's transfer entity.
    """
    if entity is None:
        entity = rset.get_entity(0, 0)
    version = getattr(cls, 'seda_version', kwargs.get('version', req.form.get('version', '2.0')))
    if version != '2.0' and not entity.simplified_profile:
        return 0
    return int(('SEDA ' + version) in entity.formats_compat)


class SEDADownloadAction(action.Action):
    __abstract__ = True
    __select__ = (action.Action.__select__
                  & one_line_rset()
                  & is_instance('SEDAArchiveTransfer')
                  & format_supported())
    category = 'moreactions'

    # set those in concret classes
    __regid__ = title = seda_version = export_format = order = None

    def url(self):
        entity = self.cw_rset.get_entity(self.cw_row or 0, self.cw_col or 0)
        return entity.absolute_url(vid='seda.export',
                                   version=self.seda_version,
                                   format=self.export_format)


class SEDA2DownloadRNGAction(SEDADownloadAction):
    title = _('SEDA 2.0 RNG export')
    order = 100
    seda_version = '2.0'
    export_format = 'rng'
    __regid__ = 'seda.export.{0}.{1}'.format(seda_version, export_format)


class SEDA1DownloadRNGAction(SEDADownloadAction):
    title = _('SEDA 1.0 RNG export')
    order = 102
    seda_version = '1.0'
    export_format = 'rng'
    __regid__ = 'seda.export.{0}.{1}'.format(seda_version, export_format)


class SEDA02DownloadRNGAction(SEDADownloadAction):
    title = _('SEDA 0.2 RNG export')
    order = 104
    seda_version = '0.2'
    export_format = 'rng'
    __regid__ = 'seda.export.{0}.{1}'.format(seda_version, export_format)


class SEDA2DownloadHTMLAction(SEDADownloadAction):
    title = _('HTML documentation')
    order = 110
    seda_version = '2.0'
    export_format = 'html'
    __regid__ = 'seda.export.{0}.{1}'.format(seda_version, export_format)


class SEDAExportViewMixin(object):
    __regid__ = 'seda.export'
    __select__ = one_line_rset() & is_instance('SEDAArchiveTransfer')


class SEDAExportView(SEDAExportViewMixin, idownloadable.DownloadView):
    """SEDA archive transfer export view, to download rng, html or xsd representation of a profile.
    """
    __select__ = SEDAExportViewMixin.__select__ & format_supported()

    http_cache_manager = httpcache.NoHTTPCacheManager

    @property
    def seda_adapter_id(self):
        return 'SEDA-{0}.{1}'.format(
            self.cw_extra_kwargs.get('version', self._cw.form.get('version', '2.0')),
            self.cw_extra_kwargs.get('format', self._cw.form.get('format', 'rng')).lower())

    def set_request_content_type(self):
        entity = self.cw_rset.get_entity(self.cw_row or 0, self.cw_col or 0)
        adapter = entity.cw_adapt_to(self.seda_adapter_id)
        filename = '%s-%s.%s' % (entity.dc_title(),
                                 self._cw.form.get('version', '2.0'),
                                 self._cw.form.get('format', 'rng').lower())
        self._cw.set_content_type(adapter.content_type, filename=filename,
                                  encoding=adapter.encoding, disposition='attachment')

    def call(self, **kwargs):
        entity = self.cw_rset.get_entity(self.cw_row or 0, self.cw_col or 0)
        adapter = entity.cw_adapt_to(self.seda_adapter_id)
        self.w(adapter.dump())


class SEDANonSupportedExportView(SEDAExportViewMixin, view.EntityView):
    """SEDA archive transfer export view for cases where an unsupported export is specified.
    """
    __select__ = SEDAExportViewMixin.__select__ & ~format_supported()

    def entity_call(self, entity):
        self.w(u'<p class="bg-error">')
        version = self.cw_extra_kwargs.get('version', self._cw.form.get('version', '2.0'))
        self.w(self._cw._('Export to SEDA version {0} is not possible.').format(version))
        self.w(u'</p>')
        self.w(u'<p>')
        self.w(self._cw._('Supported formats: {0}.').format(entity.compat_list))
        self.w(u'</p>')



# copyright 2016-2017 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""Miscellaneous test utilities."""

from contextlib import contextmanager

from cubicweb import ValidationError, Unauthorized
from cubicweb.rset import NoResultError


@contextmanager
def assertValidationError(self, cnx):
    with self.assertRaises(ValidationError) as cm:
        yield cm
        cnx.commit()
    cnx.rollback()


@contextmanager
def assertUnauthorized(self, cnx):
    with self.assertRaises(Unauthorized) as cm:
        yield cm
        cnx.commit()
    cnx.rollback()


def create_transfer_to_bdo(cnx):
    """Create minimal ArchiveTransfer down to a BinaryDataObject and return the later."""
    transfer = cnx.create_entity('SEDAArchiveTransfer', title=u'test profile')
    bdo = create_data_object(transfer)
    # commit and clear cache to allow access to container relation
    cnx.commit()
    bdo.cw_clear_all_caches()
    return bdo


def create_archive_unit(parent, archive_unit_reference=False, title=None, **kwargs):
    """Create an archive unit and its mandatory children:

    return (archive unit, alternative, sequence) if archive_unit_reference is false (the default),
    else (archive unit, alternative, reference).
    """
    cnx = kwargs.pop('cnx', getattr(parent, '_cw', None))
    kwargs.setdefault('user_annotation', u'archive unit title')
    au = cnx.create_entity('SEDAArchiveUnit', seda_archive_unit=parent, **kwargs)
    alt = cnx.create_entity('SEDAAltArchiveUnitArchiveUnitRefId',
                            reverse_seda_alt_archive_unit_archive_unit_ref_id=au)
    if archive_unit_reference:
        last = cnx.create_entity('SEDAArchiveUnitRefId', seda_archive_unit_ref_id_from=alt)
    else:
        try:
            description_level = cnx.execute('Concept C WHERE L label_of C, L label "file"').one()
        except NoResultError:
            with cnx.security_enabled(write=False):
                scheme = scheme_for_type(cnx, 'seda_description_level', None, u'file')
                description_level = scheme.reverse_in_scheme[0]
        last = cnx.create_entity(
            'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement',
            reverse_seda_seq_alt_archive_unit_archive_unit_ref_id_management=alt,
            seda_description_level=description_level)
        cnx.create_entity('SEDATitle', seda_title=last, title=title)

    return au, alt, last


def create_data_object(parent, **kwargs):
    cnx = getattr(parent, '_cw', None)
    if parent.cw_etype == 'SEDAArchiveTransfer':
        kwargs['seda_binary_data_object'] = parent
    kwargs.setdefault('user_annotation', u'data object title')
    bdo = cnx.create_entity('SEDABinaryDataObject', **kwargs)
    choice = cnx.create_entity('SEDAAltBinaryDataObjectAttachment',
                               reverse_seda_alt_binary_data_object_attachment=bdo)
    cnx.create_entity('SEDAAttachment', seda_attachment=choice)  # Choice cannot be empty
    if parent.cw_etype != 'SEDAArchiveTransfer':
        cnx.create_entity('SEDADataObjectReference', user_cardinality=u'0..n',
                          seda_data_object_reference=parent,
                          seda_data_object_reference_id=bdo)

    return bdo


def create_authority_record(cnx, name, kind=u'person', **kwargs):
    """Return an AuthorityRecord with specified kind and name."""
    kind_eid = cnx.find('AgentKind', name=kind)[0][0]
    record = cnx.create_entity('AuthorityRecord', agent_kind=kind_eid, **kwargs)
    cnx.create_entity('NameEntry', parts=name, form_variant=u'authorized',
                      name_entry_for=record)
    return record


def map_cs_to_type(scheme, rtype, etype=None):
    cnx = scheme._cw
    cnx.execute('SET CS scheme_relation_type RT WHERE CS eid %(cs)s, RT name %(rt)s',
                {'cs': scheme.eid, 'rt': rtype})
    if etype is not None:
        cnx.execute('SET CS scheme_entity_type ET WHERE CS eid %(cs)s, ET name %(et)s',
                    {'cs': scheme.eid, 'et': etype})


# extra kwargs useful for client lib (e.g. saem)
def scheme_for_type(cnx, rtype, etype, *concept_labels, **kwargs):
    """Create a concept scheme an map it to give rtype and optional etype."""
    scheme = cnx.create_entity('ConceptScheme', title=u'{0}/{1} vocabulary'.format(rtype, etype),
                               **kwargs)
    map_cs_to_type(scheme, rtype, etype)
    for label in concept_labels:
        scheme.add_concept(label)
    return scheme


def scheme_for_rtype(cnx, rtype, *concept_labels):
    return scheme_for_type(cnx, rtype, None, *concept_labels)



# copyright 2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-seda unit tests for dataimport"""

from cubicweb.devtools.testlib import CubicWebTC

from cubicweb_seda import dataimport


class ConcepSchemeImportTC(CubicWebTC):

    def test_import_seda_schemes(self):
        with self.admin_access.cnx() as cnx:
            dataimport.import_seda_schemes(cnx, lcsv_import=dataimport.lcsv_check)
            self.assertEqual(len(cnx.find('ConceptScheme')), 20)


if __name__ == '__main__':
    import unittest
    unittest.main()



from __future__ import print_function

from cubicweb_seda import seda_profile_container_def, iter_all_rdefs

for etype, parent_rdefs in seda_profile_container_def(schema):
    sync_schema_props_perms(etype, syncprops=False)

for rdef, role in iter_all_rdefs(schema, 'SEDAArchiveTransfer'):
    try:
        fsschema[rdef.rtype].rdefs[(rdef.subject, rdef.object)]
    except KeyError:
        print('junk detected in database schema', rdef)
    else:
        sync_schema_props_perms((str(rdef.subject), str(rdef.rtype), str(rdef.object)),
                                syncprops=False)
